
/*!
 * castaneum
 * Copright(c) 2010 goddamnbugs.com
 * MIT Licensed
 */

exports.version = '0.2.1';

var assert = require('assert'),
    sys = require('sys'),
    events = require('events'),
    http = require('http'),
    url = require('url');

var sizzle = require("../../deps/node-sizzle/node-sizzle"),
    jsdom = require("jsdom");

assert.ok(!!sizzle, "sizzle module.");
assert.ok(!!jsdom, "jsdom module.");

var Browser = exports.Browser = function() {
  this.specs = new Array();

  // this is weird...also using arguments without checking stuff...my bad for now...
//  var spec = arguments[0][0];
//  if (spec) {
//    var that = this;
//    this.load(url.parse(spec.url), function(window, document, $) {
//      that.title = document.title;
//      that.location = document.location;
//      that.posts = []
//      $(spec.post.title).forEach(function(el) {
//        var post = { 'title' : el.text };
//        that.posts.push(post);
//      });

//      that.emit('loaded');
//    });
//  }
}; 

// Browser is a subclass of events.EventEmitter
sys.inherits(Browser, events.EventEmitter);

exports.createBrowser = function(spec) {
  return new Browser(Array.prototype.slice.call(arguments));
};


Browser.prototype.readSpecSync = function(specPath) {
  var data = require('fs').readFileSync(specPath, 'utf8');
  return JSON.parse(data);
};


Browser.prototype.loadSpecs = function(specPath) {
  var that = this;
  var files = require('fs').readdirSync(specPath);
    files.forEach(function(file) {
      var filePath = require('path').join(specPath, file);
      var spec = that.readSpecSync(filePath);
      var specKey = require('path').basename(file, '.json');
      that.specs[specKey] = spec;
    });
};

Browser.prototype.load = function(target, callback) {
  var window;
  if (typeof(target) == 'string') {
    target = url.parse(target);
  }
  if (typeof(target) == "object") {
    if (target.protocol && target.protocol == "http:") {
      var tokens = target.hostname.split('.');
      var host = tokens.length == 3 ? tokens[1] : tokens[0];
      var spec = undefined;
      if (this.specs && host in this.specs) {
        spec = this.specs[host];
      }
      this.get(target, function(err, data, options) {
        window = jsdom.jsdom(data).createWindow();
        assert.ok(!!window, 'jsdom window');
        assert.ok(!!window.document, 'jsdom window.document');
        var $ = sizzle.loadSizzle(window.document);
        assert.ok(!!$, 'sizzle $');
        window.document.location = url.format(target);
        window.document.posts = [];
        if (spec) {
          $(spec.post.title).forEach(function(el) {
            var post = { 'title' : el.text };
            window.document.posts.push(post);
          });
          $(spec.gallery.url).forEach(function(el) {
            var href = el.href;
            if (/jpg$/.test(href)) {
            console.log(href);
              if (/^\//.test(href)) {
                console.log('absolute');
              } else if (/^http/.test(href)) {
                console.log('full url');
              } else {
                var itemurl = require('path').join(url.format(target), href).replace(':/', '://');
                console.log(itemurl);
              }
            }
          });
        }
        callback(window, window.document, $);
      });
    }
  } else {
    window = jsdom.jsdom(target).createWindow();
    callback(window, window.document, sizzle.loadSizzle(window.document));
  }
};

Browser.prototype.get = function(urlObj, callback) {
  var port = urlObj.port || 80;
  var client = http.createClient(port, urlObj.hostname);
  if (urlObj.query) {
    var req = client.request('GET', urlObj.pathname + '?' + urlObj.query, {'host':urlObj.hostname});
  } else {
    var req = client.request('GET', urlObj.pathname, {'host':urlObj.hostname});
  }
  req.on('response', function(res) {
    var chunks = [],
        i = 0;
    res.on('data', function(chunk) {
      chunks[i++] = chunk;
    });
    res.on('end', function() {
      var err = undefined;
      var data = chunks.join('');
      // node-htmlparser does not like '>>' or '<<'...fix them here rather than mess with the parser
      data = data.replace('>>', '>').replace('<<', '<');
      var options = [];
      options['headers'] = JSON.stringify(res.headers);
      callback(err, data, options);
    });
  });
  req.end();
};

exports.loadSizzle = function(data) {
//  var dom = require('./deps/jsdom/lib/jsdom/level2/core').dom;
//  assert.ok(!!dom, "dom module");
//  var level = dom.level2.core;
//  assert.ok(!!level, "level issue");

  // node-htmlparser does not like '>>' or '<<'...change them to continue rather than mess with the parser for now
  data = data.replace('<<', '<').replace('>>', '>');
  var window = jsdom.jsdom(data).createWindow();
  return sizzle.loadSizzle(window.document);  
};

